﻿-- ********************************************************************************
-- Broker Statistics
-- A Data Broker display for combat statistics.
-- By: Shenton
--
-- Alert.lua
-- Alert when the specfied statistics reach a specified amount.
-- ********************************************************************************

local A = _G["DataBrokerStatisticsGlobal"];

-- Lua globals
local ipairs = ipairs;

-- ********************************************************************************
-- Variables & constants
-- ********************************************************************************

-- Frames container
local alertFrames = {};

-- Frames count
local alertFramesCount = 0;

-- Alerts container
local alerts = {};

-- Alerts status table
local alertsStatus = {};

-- ********************************************************************************
-- Functions
-- ********************************************************************************

--- Create an alert frame
-- @param name The alert name
function A:CreateAlertFrame(name)
    alertFramesCount = alertFramesCount + 1;

    local f = CreateFrame("Frame", "DataBrokerStatisticsAlert"..alertFramesCount);
    alertFrames[#alertFrames+1] = f;
    f.alertName = name;
    f.texFrame = CreateFrame("Frame");
    f.texFrame:SetPoint("CENTER", f);
    f.texture = f.texFrame:CreateTexture();
    f.texture:SetAllPoints(f.texFrame);

    f:SetScript("OnDragStart", f.StartMoving);
    f:SetScript("OnDragStop", function(self)
        self:StopMovingOrSizing();

        local point, relativeTo, relativePoint, x, y = self:GetPoint(1);

        A.db.global.alerts[name].point = point;
        A.db.global.alerts[name].parent = relativeTo;
        A.db.global.alerts[name].relative = relativePoint;
        A.db.global.alerts[name].offX = x;
        A.db.global.alerts[name].offY = y;
    end);

    return f;
end

--- Get an alert frame from the stack
-- @param name Alert name
-- @param anim Animation internal name
function A:GetAlertFrame(name, anim)
    for _,v in ipairs(alertFrames) do
        if ( v.alertName == name and v.animType == anim ) then
            return v;
        end
    end

    return nil;
end

--- Hide alert frames
-- @param name Alert name
function A:HideAlertFrames(name)
    for _,v in ipairs(alertFrames) do
        if ( v.alertName == name ) then
            v:Hide();
            v.texFrame:Hide();
            if ( v.animGroup ) then v.animGroup:Stop(); end
        end
    end
end

--- Hide alerts frames
-- function A:HideAlertsFrames()
    -- for _,v in ipairs(alertFrames) do
        -- v:Hide();
        -- v.texFrame:Hide();
        -- if ( v.animGroup ) then v.animGroup:Stop(); end
    -- end
-- end

--- Set an alert frame
-- @param name Alert name
-- @param db Alert options
function A:SetAlertFrame(name, db)
    local f = A:GetAlertFrame(name, db.anim);

    A:HideAlertFrames(name);

    if ( not f ) then
        f = A:CreateAlertFrame(name);
        A.alertAnim[db.anim](f);
    end

    f:SetSize(db.x + 10, db.y + 10);
    f:ClearAllPoints();
    f:SetPoint(db.point, db.parent, db.relative, db.offX, db.offY);
    f.texFrame:SetSize(db.x, db.y);
    f.texFrame:SetAlpha(1);
    f.texture:SetTexture(A.alertTextures[db.texture]);
    f.texture:SetBlendMode(db.alphaMode);
    f.texture:SetVertexColor(db.r, db.g, db.b, db.a);

    if ( f.animGroup ) then
        f.texFrame:SetScript("OnShow", function()
            f.animGroup:Play();
        end);
        f.texFrame:SetScript("OnHide", function()
            f.animGroup:Stop();
        end);
    else
        f.texFrame:SetScript("OnShow", nil);
        f.texFrame:SetScript("OnHide", nil);
    end

    if ( db.alertConfigMode ) then
        f:EnableMouse(1);
        f:SetMovable(1);
        f:RegisterForDrag("LeftButton");
        f:SetBackdrop({ edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16 });
        f:Show();
        f.texFrame:Show();
        if ( f.animGroup ) then f.animGroup:Play(); end
    else
        f:EnableMouse(nil);
        f:SetMovable(nil);
        f:RegisterForDrag(nil);
        f:SetBackdrop({ edgeFile = nil });
        f:Hide();
        f.texFrame:Hide();
        if ( f.animGroup ) then f.animGroup:Stop(); end
    end

    return f;
end

--- Set an alert (sound & frame)
-- @param name Alert name
function A:SetAlert(name)
    if ( A.alertTest ) then return; end

    local db = A.db.global.alerts[name];
    local lsmSounds = A.LibSharedMedia:HashTable("sound");
    local sound = lsmSounds[db.sound];

    if ( not sound ) then
        sound = lsmSounds["None"];
        A:Message(A.L["Sound %s not found for alert %s."]:format(db.sound, name), 1, 1);
    end

    if ( db.alertConfigMode ) then
        local frame = A:SetAlertFrame(name, db);

        alerts[name] = function(hide)
        end;

        return;
    end

    if ( db.enableTexture and db.enableSound ) then
        local frame = A:SetAlertFrame(name, db);

        alerts[name] = function(hide)
            if ( hide ) then
                if ( frame.texFrame:IsVisible() ) then
                    frame.texFrame:Hide();
                end
            else
                if ( not frame.texFrame:IsVisible() ) then
                    frame.texFrame:Show();
                    PlaySoundFile(sound, "Master");
                end
            end
        end;
    elseif ( db.enableTexture ) then
        local frame = A:SetAlertFrame(name, db);

        alerts[name] = function(hide)
            if ( hide ) then
                if ( frame.texFrame:IsVisible() ) then
                    frame.texFrame:Hide();
                end
            else
                if ( not frame.texFrame:IsVisible() ) then
                    frame.texFrame:Show();
                end
            end
        end;
    elseif ( db.enableSound ) then
        alerts[name] = function(hide)
            if ( not hide ) then
                PlaySoundFile(sound, "Master");
            end
        end;
    else
        alerts[name] = function(hide)
        end;
    end
end

--- Fire an alert
-- @param name Alert name
-- @param hide Bool if true hide the frame
-- @param timer Time before hidding the frame
function A:Alert(name, hide, timer)
    if ( timer ) then
        if ( hide ) then
            alertsStatus[name] = nil;
        elseif ( not alertsStatus[name] ) then
            alerts[name]();
            A:ScheduleTimer(alerts[name], timer, 1);
            alertsStatus[name] = 1;
        end
    else
        if ( hide ) then
            alerts[name](1);
        else
            alerts[name]();
        end
    end
end

--- Fire an alert for testing purpose
-- @param name Alert name
function A:AlertTest(name)
    A:Pause();
    A.alertTest = A:ScheduleTimer("AlertTestTimer", 5, name);
    alerts[name]();
end

--- Test alert timer
-- @param name Alert name
function A:AlertTestTimer(name)
    alerts[name](1);
    A.alertTest = nil;
    A:Unpause();
end

-- ********************************************************************************
-- Animations
-- ********************************************************************************

local function AlertAnimNone(f)
    f.animType = "none";
end

local function AlertAnimBlink(f)
    f.animGroup = f.texFrame:CreateAnimationGroup();
    f.animGroup:SetLooping("BOUNCE");
    f.animType = "blink";
    local anim = f.animGroup:CreateAnimation("Alpha");
    anim:SetChange(-0.6);
    anim:SetDuration(0.5);
    anim:SetOrder(1);
end

local function AlertAnimPulseShrink(f)
    f.animGroup = f.texFrame:CreateAnimationGroup();
    f.animGroup:SetLooping("BOUNCE");
    f.animType = "pulses";
    local anim = f.animGroup:CreateAnimation("Scale");
    anim:SetScale(0.8, 0.8);
    anim:SetDuration(0.2);
    anim:SetOrder(1);
end

local function AlertAnimPulseEnlarge(f)
    f.animGroup = f.texFrame:CreateAnimationGroup();
    f.animGroup:SetLooping("BOUNCE");
    f.animType = "pulsee";
    local anim = f.animGroup:CreateAnimation("Scale");
    anim:SetScale(1.2, 1.2);
    anim:SetDuration(0.2);
    anim:SetOrder(1);
end

local function AlertAnimRotateClockwise(f)
    f.animGroup = f.texFrame:CreateAnimationGroup();
    f.animGroup:SetLooping("REPEAT");
    f.animType = "rotatec";
    local anim = f.animGroup:CreateAnimation("Rotation");
    anim:SetDegrees(-360);
    anim:SetDuration(1.5);
    anim:SetOrder(1);
end

local function AlertAnimRotateAntiClockwise(f)
    f.animGroup = f.texFrame:CreateAnimationGroup();
    f.animGroup:SetLooping("REPEAT");
    f.animType = "rotateac";
    local anim = f.animGroup:CreateAnimation("Rotation");
    anim:SetDegrees(360);
    anim:SetDuration(1.5);
    anim:SetOrder(1);
end

local function AlertAnimShakeHorizontal(f)
    f.animGroup = f.texFrame:CreateAnimationGroup();
    f.animGroup:SetLooping("BOUNCE");
    f.animType = "shakeh";
    local a = f.animGroup:CreateAnimation("Translation");
    a:SetDuration(0.2);
    a:SetOffset(5, 0);
    a:SetOrder(1);
    local b = f.animGroup:CreateAnimation("Translation");
    b:SetDuration(0.2);
    b:SetOffset(-5, 0);
    b:SetOrder(2);
end

local function AlertAnimShakeVertical(f)
    f.animGroup = f.texFrame:CreateAnimationGroup();
    f.animGroup:SetLooping("BOUNCE");
    f.animType = "shakev";
    local a = f.animGroup:CreateAnimation("Translation");
    a:SetDuration(0.2);
    a:SetOffset(0, 5);
    a:SetOrder(1);
    local b = f.animGroup:CreateAnimation("Translation");
    b:SetDuration(0.2);
    b:SetOffset(0, -5);
    b:SetOrder(2);
end

A.alertAnim =
{
    none = AlertAnimNone,
    blink = AlertAnimBlink,
    pulses = AlertAnimPulseShrink,
    pulsee = AlertAnimPulseEnlarge,
    rotatec = AlertAnimRotateClockwise,
    rotateac = AlertAnimRotateAntiClockwise,
    shakeh = AlertAnimShakeHorizontal,
    shakev = AlertAnimShakeVertical,
};
