-- ********************************************************************************
-- Broker Statistics
-- A Data Broker display for combat statistics.
-- By: Shenton
--
-- Core.lua
-- Ace3 configuration panel and databases.
-- ********************************************************************************

local A = _G["DataBrokerStatisticsGlobal"];

-- Ace Libs (<3)
A.AceDBOptions = LibStub("AceDBOptions-3.0");
A.AceConfigDialog = LibStub("AceConfigDialog-3.0");
A.AceConfigRegistry = LibStub("AceConfigRegistry-3.0");

-- Lua globals
local loadstring = loadstring;

-- WoW globals
local strtrim = strtrim;

-- ********************************************************************************
-- Constants
-- ********************************************************************************

-- Lua globals
local format, pairs, smatch, strlen = format, pairs, string.match, string.len;
local tonumber, tostring = tonumber, tostring;

-- Alert alpha mode select options
local alphamodeSelect =
{
    DISABLE = A.L["Opaque texture"],
    BLEND = A.L["Blend"],
    ALPHAKEY = A.L["One-bit alpha"],
    ADD = A.L["Additive blend"],
    MOD = A.L["Modulating blend"],
};

-- Alert animations select options
local animationSelect =
{
    none = A.L["None"],
    blink = A.L["Blink"],
    pulses = A.L["Pulse Shrink"],
    pulsee = A.L["Pulse Enlarge"],
    rotatec = A.L["Rotate Clockwise"],
    rotateac = A.L["Rotate Anti Clockwise"],
    shakeh = A.L["Shake Horizontal"],
    shakev = A.L["Shake Vertical"],
};

-- Alert config select
local configAlertSelect =
{
    none = A.L["None"],
    texture = A.L["Texture"],
    sound = A.L["Sound"],
};

-- Custom stats config select
local configCustomSelect =
{
    none = A.L["None"],
    alert = A.L["Alert"],
    options = A.L["Options"],
    order = A.L["Order"],
    share = A.L["Share"],
};

-- Built in config select
local configBuiltInSelect =
{
    none = A.L["None"],
    alert = A.L["Alert"],
    display = A.L["Display"],
    order = A.L["Order"],
};

-- Alert mode select
local configAlertModeSelect =
{
    normal = A.L["Normal"],
    proc = A.L["Proc"],
};

-- ********************************************************************************
-- Configuration
-- ********************************************************************************

--- AceConfig menu
local options = {};
local addName, addString = nil, nil;
local deleteCustom, deleteDbObject, deleteAlert = {}, {}, {};
local buitInConfig, customConfig, alertConfig = {}, {}, {};
local orderSelect, orderDescription;
function A:AceConfig()
    orderSelect = A:CreateOrderSelect();
    orderDescription = A:CreateOrderDescriptions();

    options =
    {
        name = A.L["Broker Statistics"],
        type = "group",
        childGroups = "tab",
        args =
        {
            options =
            {
                order = 10,
                name = A.L["Options"],
                type = "group",
                args =
                {
                    main =
                    {
                        order = 0,
                        name = A.L["Main"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            verbosity =
                            {
                                order = 0,
                                name = A.L["Verbosity"],
                                desc = A.L["Set the addon chat messages verbosity."],
                                type = "select",
                                values = { [0] = A.L["Disabled"], [1] = A.L["Normal"], [2] = A.L["Everything"] },
                                set = function(info, val) A.db.profile.verbosity = val; end,
                                get = function(info) return A.db.profile.verbosity; end,
                            },
                            refreshRate =
                            {
                                order = 1,
                                name = A.L["Refresh Rate"],
                                desc = A.L["The refresh rate in seconds."],
                                type = "range",
                                min = 0.1,
                                max = 5,
                                step = 0.1,
                                set = function(info, val)
                                    A.db.profile.time = val;
                                    A:RefreshTimer();
                                end,
                                get = function(info) return A.db.profile.time; end,
                            },
                        },
                    },
                    configurationBehavior =
                    {
                        order = 3,
                        name = A.L["Configuration save behavior"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            accordingToSpec =
                            {
                                order = 0,
                                name = A.L["According to specialization"],
                                desc = A.L["Enable if the configuration will be saved according to the current specialization.\n\nNote: This only affect Data Broker objects and statistics."],
                                type = "toggle",
                                set = function(info, val)
                                    A.db.profile.configAccordingSpec = val;
                                    A:RefreshConfig();
                                end,
                                get = function(info) return A.db.profile.configAccordingSpec; end,
                            },
                            specSaveBehavior =
                            {
                                order = 1,
                                name = A.L["Specialization save behavior"],
                                desc = A.L["Choose if the configuration will be saved just on the specialization name or the specialization name and numbers (x/x/x/x/x/x)."],
                                type = "select",
                                values = function() return { [1] = A.L["Specialization name"], [2] = A.L["Specialization name and numbers"] } end,
                                disabled = function() return not A.db.profile.configAccordingSpec; end,
                                set = function(info, val)
                                    A.db.profile.configAccordingSpecBehavior = val;
                                    A:RefreshConfig();
                                end,
                                get = function(info) return A.db.profile.configAccordingSpecBehavior; end,
                            },
                        },
                    },
                    shareReject =
                    {
                        order = 4,
                        name = A.L["Reject share request"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            whisper =
                            {
                                order = 0,
                                name = A.L["Player"],
                                desc = A.L["Reject share from other players."],
                                type = "toggle",
                                get = function() return A.db.profile.shareWhisper; end,
                                set = function(info, val) A.db.profile.shareWhisper = val; end,
                            },
                            guild =
                            {
                                order = 1,
                                name = A.L["Guild"],
                                desc = A.L["Reject share from guild."],
                                type = "toggle",
                                get = function() return A.db.profile.shareGuild; end,
                                set = function(info, val) A.db.profile.shareGuild = val; end,
                            },
                            raid =
                            {
                                order = 2,
                                name = A.L["Raid"],
                                desc = A.L["Reject share from raid or party."],
                                type = "toggle",
                                get = function() return A.db.profile.shareRaid; end,
                                set = function(info, val) A.db.profile.shareRaid = val; end,
                            },
                            bg =
                            {
                                order = 3,
                                name = A.L["Battleground"],
                                desc = A.L["Reject share from battleground."],
                                type = "toggle",
                                get = function() return A.db.profile.shareBg; end,
                                set = function(info, val) A.db.profile.shareBg = val; end,
                            },
                        },
                    },
                },
            },
            dataBroker =
            {
                order = 20,
                name = A.L["Data Broker"],
                type = "group",
                args =
                {
                    defaultDisplay =
                    {
                        order = 0,
                        name = A.L["Default object"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            selectIcon =
                            {
                                order = 0,
                                name = function()
                                    return A.L["Select Icon"]
                                end,
                                desc = A.L["Select the Data Broker object main icon.\n\n|cFFC79C6ELeft-Click:|r Default icon list (Macro frame).\n|cFFC79C6EShift+Left-Click:|r Item macro list (Guild bank frame)."],
                                desc = A.L["Select the Data Broker object main icon."],
                                type = "execute",
                                image = function(info)
                                    local icon = A.db.profile.dataBroker.defaultObject.icon or A.defaultLdbIcon;
                                    return icon, 36, 36; end,
                                func = function()
                                    local icon = A.db.profile.dataBroker.defaultObject.icon or A.defaultLdbIcon;

                                    if ( not BrokerStatisticsPopupFrame ) then A:IconsFrame(); end

                                    if ( not BrokerStatisticsPopupFrame:IsVisible() ) then
                                        BrokerStatisticsPopupFrame:ClearAllPoints();
                                        BrokerStatisticsPopupFrame:SetPoint("CENTER", GetMouseFocus(), "CENTER", 150, 0);

                                        --if ( IsShiftKeyDown() ) then BrokerStatisticsPopupFrame.iconList = 1; end
                                        BrokerStatisticsPopupFrame.currentTexture = icon;
                                        BrokerStatisticsPopupFrame.dbEdit = A.db.profile.dataBroker.defaultObject;
                                        BrokerStatisticsPopupFrame.ldbObject = A["ldbObjectDefault"];

                                        BrokerStatisticsPopupFrame:Show();
                                        A:PopupScrollUpdate();
                                    end
                                end
                            },
                            lineBreak1 =
                            {
                                order = 1,
                                name = "",
                                type = "description",
                                width = "full",
                            },
                            pre =
                            {
                                order = 2,
                                name = A.L["Pre"],
                                desc = A.L["Display the stat prefix."],
                                type = "toggle",
                                set = function(info, val) A.db.profile.defaultDisplay[A.playerSpec].displayPre = val; end,
                                get = function(info) return A.db.profile.defaultDisplay[A.playerSpec].displayPre; end,
                            },
                            post =
                            {
                                order = 3,
                                name = A.L["Post"],
                                desc = A.L["Display the stat post string."],
                                type = "toggle",
                                set = function(info, val) A.db.profile.defaultDisplay[A.playerSpec].displayPost = val; end,
                                get = function(info) return A.db.profile.defaultDisplay[A.playerSpec].displayPost; end,
                            },
                            separator =
                            {
                                order = 4,
                                name = A.L["Separator"],
                                desc = A.L["Enter the separator displayed between each stat."],
                                type = "input",
                                set = function(info, val) A.db.profile.defaultDisplay[A.playerSpec].statsSeparator = val; end,
                                get = function() return A.db.profile.defaultDisplay[A.playerSpec].statsSeparator; end,
                            },
                            lineBreak10 =
                            {
                                order = 10,
                                name = "",
                                type = "description",
                                width = "full",
                            },
                            preColor =
                            {
                                order = 11,
                                name = A.L["Pre Color"],
                                desc = A.L["Pickup a color."],
                                type = "color",
                                hasAlpha = true,
                                set = function(info, r, g, b, a)
                                    A.db.profile.defaultDisplay[A.playerSpec].preColor.r = r;
                                    A.db.profile.defaultDisplay[A.playerSpec].preColor.g = g;
                                    A.db.profile.defaultDisplay[A.playerSpec].preColor.b = b;
                                    A.db.profile.defaultDisplay[A.playerSpec].preColor.a = a;
                                    A.db.profile.defaultDisplay[A.playerSpec].preColorHex = "|c"..A:PercentToHex(r, g, b, a);
                                end,
                                get = function()
                                    return A.db.profile.defaultDisplay[A.playerSpec].preColor.r,
                                        A.db.profile.defaultDisplay[A.playerSpec].preColor.g,
                                        A.db.profile.defaultDisplay[A.playerSpec].preColor.b,
                                        A.db.profile.defaultDisplay[A.playerSpec].preColor.a;
                                end,
                            },
                            postColor =
                            {
                                order = 12,
                                name = A.L["Post Color"],
                                desc = A.L["Pickup a color."],
                                type = "color",
                                hasAlpha = true,
                                set = function(info, r, g, b, a)
                                    A.db.profile.defaultDisplay[A.playerSpec].postColor.r = r;
                                    A.db.profile.defaultDisplay[A.playerSpec].postColor.g = g;
                                    A.db.profile.defaultDisplay[A.playerSpec].postColor.b = b;
                                    A.db.profile.defaultDisplay[A.playerSpec].postColor.a = a;
                                    A.db.profile.defaultDisplay[A.playerSpec].postColorHex = "|c"..A:PercentToHex(r, g, b, a);
                                end,
                                get = function()
                                    return A.db.profile.defaultDisplay[A.playerSpec].postColor.r,
                                        A.db.profile.defaultDisplay[A.playerSpec].postColor.g,
                                        A.db.profile.defaultDisplay[A.playerSpec].postColor.b,
                                        A.db.profile.defaultDisplay[A.playerSpec].postColor.a;
                                end,
                            },
                            separatorColor =
                            {
                                order = 13,
                                name = A.L["Separator Color"],
                                desc = A.L["Pickup a color."],
                                type = "color",
                                hasAlpha = true,
                                set = function(info, r, g, b, a)
                                    A.db.profile.defaultDisplay[A.playerSpec].separatorColor.r = r;
                                    A.db.profile.defaultDisplay[A.playerSpec].separatorColor.g = g;
                                    A.db.profile.defaultDisplay[A.playerSpec].separatorColor.b = b;
                                    A.db.profile.defaultDisplay[A.playerSpec].separatorColor.a = a;
                                    A.db.profile.defaultDisplay[A.playerSpec].separatorColorHex = "|c"..A:PercentToHex(r, g, b, a);
                                end,
                                get = function()
                                    return A.db.profile.defaultDisplay[A.playerSpec].separatorColor.r,
                                        A.db.profile.defaultDisplay[A.playerSpec].separatorColor.g,
                                        A.db.profile.defaultDisplay[A.playerSpec].separatorColor.b,
                                        A.db.profile.defaultDisplay[A.playerSpec].separatorColor.a;
                                end,
                            },
                        },
                    },
                    addNew =
                    {
                        order = 1,
                        name = A.L["Add New"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            inputAdd =
                            {
                                order = 0,
                                name = A.L["Enter Name"],
                                desc = A.L["Enter the new Data Broker object name."],
                                type = "input",
                                set = function(info, val) addName = val; end,
                                get = function() return addName; end,
                            },
                            executeAdd =
                            {
                                order = 1,
                                name = A.L["Create"],
                                disabled = function() return not addName; end,
                                desc = A.L["Create the new Data Broker object %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    addName = strtrim(addName);
                                    if ( not addName or addName == "" ) then
                                        A:Message(A.L["The name provided was empty."], 1, 1);
                                        return;
                                    end

                                    if ( A.db.global.customDisplays[addName] ) then
                                        A:Message(A.L["The Data Broker object %s already exists."]:format(addName), 1, 1);
                                        addName = nil;
                                    else
                                        A.db.global.customDisplays[addName] = 1;
                                        A.db.profile.dataBroker.customDisplays[A.playerSpec][addName] =
                                        {
                                            enabled = 1,
                                            icon = A.defaultLdbIcon,
                                            statIconSize = 16,
                                            displayPre = 1,
                                            displayPost = 1,
                                            preColor = 
                                            {
                                                r = 0,
                                                g = 0.5,
                                                b = 0.5,
                                                a = 1
                                            },
                                            preColorHex = "|cff008080",
                                            postColor =
                                            {
                                                r = 1,
                                                g = 1,
                                                b = 1,
                                                a = 1
                                            },
                                            postColorHex = "|cffffffff",
                                            statsSeparator = "||",
                                            separatorColor =
                                            {
                                                r = 1,
                                                g = 1,
                                                b = 1,
                                                a = 1
                                            },
                                            separatorColorHex = "|cffffffff"
                                        };

                                        addName = nil;

                                        A:RefreshConfig();
                                    end
                                end,
                            },
                        },
                    },
                },
            },
            builtInStats =
            {
                order = 30,
                name = A.L["Built in Stats"],
                type = "group",
                childGroups = "tab",
                args =
                {
                    misc =
                    {
                        order = 0,
                        name = A.L["Miscellaneous"],
                        type = "group",
                        args = {},
                    },
                    tank =
                    {
                        order = 1,
                        name = A.L["Tank"],
                        type = "group",
                        args = {},
                    },
                    melee =
                    {
                        order = 2,
                        name = A.L["Melee"],
                        type = "group",
                        args = {},
                    },
                    ranged =
                    {
                        order = 3,
                        name = A.L["Ranged"],
                        type = "group",
                        args = {},
                    },
                    spell =
                    {
                        order = 4,
                        name = A.L["Spell"],
                        type = "group",
                        args = {},
                    },
                },
            },
            customStats =
            {
                order = 40,
                name = A.L["Custom Stats"],
                type = "group",
                args =
                {
                    addNew =
                    {
                        order = 0,
                        name = A.L["Add New"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            header1 =
                            {
                                order = 0,
                                name = A.L["Create New"],
                                type = "header",
                            },
                            inputAdd =
                            {
                                order = 1,
                                name = A.L["Enter Name"],
                                desc = A.L["Enter the new statistic name."],
                                type = "input",
                                set = function(info, val) addName = val; end,
                                get = function() return addName; end,
                            },
                            executeAdd =
                            {
                                order = 2,
                                name = A.L["Create"],
                                disabled = function() return not addName; end,
                                desc = A.L["Create the new custom statistic %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    addName = strtrim(addName);
                                    if ( not addName or addName == "" ) then
                                        A:Message(A.L["The name provided was empty."], 1, 1);
                                        return;
                                    end

                                    if ( A.db.global.custom[addName] ) then
                                        A:Message(A.L["The custom stat %s already exists."]:format(addName), 1, 1);
                                        addName = nil;
                                    elseif ( A.builtInStats[addName] ) then
                                        A:Message(A.L["A built in stat with that name already exists."], 1, 1);
                                        addName = nil;
                                    else
                                        A.db.global.custom[addName] = {};
                                        addName = nil;
                                    end
                                end,
                            },
                            header10 =
                            {
                                order = 10,
                                name = A.L["Import from string"],
                                type = "header",
                            },
                            inputAddString =
                            {
                                order = 11,
                                name = A.L["Paste String"],
                                desc = A.L["Paste here the new statistic string."],
                                type = "input",
                                multiline = 1,
                                width = "full",
                                set = function(info, val) addString = val; end,
                                get = function() return addString; end,
                            },
                            executeAddString =
                            {
                                order = 12,
                                name = A.L["Create"],
                                disabled = function() return not addString; end,
                                desc = A.L["Create the new custom statistic %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    local customName = smatch(addString, "^(.+){");
                                    local customText = smatch(addString, "{(.+)}");
                                    customText = "{"..customText.."}";

                                    if ( A.db.global.custom[customName] ) then
                                        local pop = StaticPopup_Show("BrokerStatsAddCustomNameExists");
                                        if (pop) then pop.customText = customText; end
                                        addString = nil;
                                    elseif ( A.builtInStats[customName] ) then
                                        local pop = StaticPopup_Show("BrokerStatsAddCustomNameExists");
                                        if (pop) then pop.customText = customText; end
                                        addString = nil;
                                    else
                                        local customTable = loadstring("return "..customText);
                                        customTable = customTable();

                                        if ( customName ) then
                                            A.db.global.custom[customName] = customTable;
                                            if ( A.AceConfigRegistry ) then A.AceConfigRegistry:NotifyChange("BrokerStatsConfig"); end
                                            addString = nil;
                                        else
                                            A:Message(A.L["The name provided was empty."], 1, 1);
                                        end
                                    end
                                end,
                            },
                        },
                    },
                },
            },
            alerts =
            {
                order = 60,
                name = A.L["Alerts"],
                type = "group",
                args =
                {
                    addNew =
                    {
                        order = 0,
                        name = A.L["Add New"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            inputAdd =
                            {
                                order = 0,
                                name = A.L["Enter Name"],
                                desc = A.L["Enter the new alert name."],
                                type = "input",
                                set = function(info, val) addName = val; end,
                                get = function() return addName; end,
                            },
                            executeAdd =
                            {
                                order = 1,
                                name = A.L["Create"],
                                disabled = function() return not addName; end,
                                desc = A.L["Create the new alert %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    addName = strtrim(addName);
                                    if ( not addName or addName == "" ) then
                                        A:Message(A.L["The name provided was empty."], 1, 1);
                                        return;
                                    end

                                    if ( A.db.global.alerts[addName] ) then
                                        A:Message(A.L["The alert %s already exists."]:format(addName), 1, 1);
                                        addName = nil;
                                    elseif ( addName == "none" ) then
                                        A:Message(A.L["The alert name cannot be \"none\"."], 1, 1);
                                        addName = nil;
                                    else
                                        A.db.global.alerts[addName] =
                                        {
                                            enableTexture = nil,
                                            texture = 1,
                                            x = 128,
                                            y = 128,
                                            point = "CENTER",
                                            parent = nil,
                                            relative = nil,
                                            offX = nil,
                                            offY = nil,
                                            alphaMode = "BLEND",
                                            anim = "none",
                                            r = 1,
                                            g = 1,
                                            b = 1,
                                            a = 1,
                                            enableSound = nil,
                                            sound = "None",
                                        };
                                        addName = nil;
                                    end
                                end
                            },
                        },
                    },
                },
            },
            specializationProfiles =
            {
                order = 70,
                name = A.L["Specialization Profiles"],
                type = "group",
                args =
                {
                    profilesDescriptions =
                    {
                        order = 0,
                        name = A.L["Here you can use, copy or delete an existing specialization profile.\nThese profiles affects Data Broker objects configuration and statistics configuration.\n\nGlobal profiles (next tab) affects everything in the configuration panel (Exept custom statistics core options which are saved in the global scope)."],
                        type = "description",
                    },
                    blankLine1 =
                    {
                        order = 1,
                        name = "",
                        type = "description",
                        width = "full",
                    },
                    activeProfile =
                    {
                        order = 2,
                        name = A.L["\nActive specialization profile: |cffffd100%s|r"]:format(A.playerSpec),
                        type = "description",
                    },
                    blankLine2 =
                    {
                        order = 3,
                        name = "",
                        type = "description",
                        width = "full",
                    },
                    specProfile =
                    {
                        order = 4,
                        name = function()
                            local spec = A:GetSpecVar();

                            return A.L["Current specialization default profile: |cffffd100%s|r"]:format(spec);
                        end,
                        type = "description",
                    },
                    useHeader =
                    {
                        order = 10,
                        name = A.L["Use"],
                        type = "header",
                    },
                    useSelect =
                    {
                        order = 11,
                        name = A.L["Use"],
                        desc = function()
                            return A.L["Select a specialization profile to use instead of %s"]:format(A.playerSpec);
                        end,
                        type = "select",
                        disabled = function() return not A.db.profile.configAccordingSpec; end,
                        values = A:CreateSpecProfilesSelect(),
                        get = function()
                            local spec = A:GetSpecVar();

                            if ( not A.db.profile.specProfiles[spec] ) then
                                A.db.profile.specProfiles[spec] = {};
                            end

                            return A.db.profile.specProfiles[spec].useSpec;
                        end,
                        set = function(info, val)
                            local spec = A:GetSpecVar();

                            if ( val == spec ) then
                                A.db.profile.specProfiles[spec].useSpec = nil;
                            else
                                A.db.profile.specProfiles[spec].useSpec = val;
                            end

                            A:RefreshConfig();
                        end,
                    },
                    copyHeader =
                    {
                        order = 20,
                        name = A.L["Copy"],
                        type = "header",
                    },
                    copySelect =
                    {
                        order = 21,
                        name = A.L["Copy"],
                        desc = A.L["Copy an existing specialization profile over the active one."],
                        type = "select",
                        confirm = function()
                            return A.L["Copy the selected profile over %s?"]:format(A.playerSpec);
                        end,
                        values = A:CreateSpecProfilesSelect(),
                        get = function() return nil; end,
                        set = function(info, val)
                            if ( val ~= A.playerSpec ) then
                                local dest = A.db.profile.dataBroker.customDisplays[A.playerSpec];
                                local src = A.db.profile.dataBroker.customDisplays[val];
                                A:CopyTable(src, dest);

                                dest = A.db.profile.builtInStats[A.playerSpec];
                                src = A.db.profile.builtInStats[val];
                                A:CopyTable(src, dest);

                                dest = A.db.profile.custom[A.playerSpec];
                                src = A.db.profile.custom[val];
                                A:CopyTable(src, dest);
                            end

                            A:RefreshConfig();
                        end,
                    },
                    deleteHeader =
                    {
                        order = 30,
                        name = A.L["Delete"],
                        type = "header",
                    },
                    deleteDesc =
                    {
                        order = 31,
                        name = A.L["Deleting the active profile will reset it."],
                        type = "description",
                    },
                    deleteSelect =
                    {
                        order = 32,
                        name = A.L["Delete"],
                        desc = A.L["Delete an existing specialization profile."],
                        type = "select",
                        confirm = function()
                            return A.L["Delete the selected profile?"];
                        end,
                        values = A:CreateSpecProfilesSelect(),
                        get = function() return nil; end,
                        set = function(info, val)
                            A.db.profile.dataBroker.customDisplays[val] = nil;
                            A.db.profile.builtInStats[val] = nil;
                            A.db.profile.custom[val] = nil;

                            A:RefreshConfig();
                        end,
                    },
                },
            },
        },
    };

    if ( A:GotCustomLdb() ) then
        options.args.dataBroker.args.list =
        {
            order = 2,
            name = A.L["Custom objects"],
            type = "group",
            inline = true,
            args = {},
        };
    end

    local order = 0;
    for k,v in A:PairsByKeys(A.db.global.customDisplays) do
        options.args.dataBroker.args.list.args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                enabled =
                {
                    order = 0,
                    name = A.L["Enable"],
                    desc = A.L["Enable this Data Broker object."],
                    type = "toggle",
                    set = function(info, val)
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].enabled = val;

                        if ( not val ) then
                            StaticPopup_Show("BrokerStatsDisablingLdbObject");
                        end

                        A:RefreshConfig();
                    end,
                    get = function(info)
                        if ( not A.db.profile.dataBroker.customDisplays[A.playerSpec][k] ) then
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k] =
                            {
                                enabled = nil,
                                icon = A.defaultLdbIcon,
                                displayPre = 1,
                                displayPost = 1,
                                preColor = 
                                {
                                    r = 0,
                                    g = 0.5,
                                    b = 0.5,
                                    a = 1
                                },
                                preColorHex = "|cff008080",
                                postColor =
                                {
                                    r = 1,
                                    g = 1,
                                    b = 1,
                                    a = 1
                                },
                                postColorHex = "|cffffffff",
                                statsSeparator = "||",
                                separatorColor =
                                {
                                    r = 1,
                                    g = 1,
                                    b = 1,
                                    a = 1
                                },
                                separatorColorHex = "|cffffffff"
                            };
                        end

                        return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].enabled;
                    end
                },
                selectIcon =
                {
                    order = 1,
                    name = function()
                        return A.L["Select Icon"]
                    end,
                    desc = A.L["Select the Data Broker object main icon.\n\n|cFFC79C6ELeft-Click:|r Default icon list (Macro frame).\n|cFFC79C6EShift+Left-Click:|r Item macro list (Guild bank frame)."],
                    desc = A.L["Select the Data Broker object main icon."],
                    type = "execute",
                    image = function(info) return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].icon, 36, 36; end,
                    func = function()
                        if ( not BrokerStatisticsPopupFrame ) then A:IconsFrame(); end

                        if ( not BrokerStatisticsPopupFrame:IsVisible() ) then
                            BrokerStatisticsPopupFrame:ClearAllPoints();
                            BrokerStatisticsPopupFrame:SetPoint("CENTER", GetMouseFocus(), "CENTER", 150, 0);

                            --if ( IsShiftKeyDown() ) then BrokerStatisticsPopupFrame.iconList = 1; end
                            BrokerStatisticsPopupFrame.currentTexture = A.db.profile.dataBroker.customDisplays[A.playerSpec][k].icon;
                            BrokerStatisticsPopupFrame.dbEdit = A.db.profile.dataBroker.customDisplays[A.playerSpec][k];
                            BrokerStatisticsPopupFrame.ldbObject = A["ldbObject"..k];

                            BrokerStatisticsPopupFrame:Show();
                            A:PopupScrollUpdate();
                        end
                    end
                },
                header10 =
                {
                    order = 10,
                    name = A.L["Display"],
                    type = "header"
                },
                pre =
                {
                    order = 11,
                    name = A.L["Pre"],
                    desc = A.L["Display the stat prefix."],
                    type = "toggle",
                    set = function(info, val) A.db.profile.dataBroker.customDisplays[A.playerSpec][k].displayPre = val; end,
                    get = function(info) return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].displayPre; end
                },
                post =
                {
                    order = 12,
                    name = A.L["Post"],
                    desc = A.L["Display the stat post string."],
                    type = "toggle",
                    set = function(info, val) A.db.profile.dataBroker.customDisplays[A.playerSpec][k].displayPost = val; end,
                    get = function(info) return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].displayPost; end
                },
                separator =
                {
                    order = 13,
                    name = A.L["Separator"],
                    desc = A.L["Enter the separator displayed between each stat."],
                    type = "input",
                    set = function(info, val) A.db.profile.dataBroker.customDisplays[A.playerSpec][k].statsSeparator = val; end,
                    get = function() return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].statsSeparator; end
                },
                lineBreak20 =
                {
                    order = 20,
                    name = "",
                    type = "description",
                    width = "full"
                },
                preColor =
                {
                    order = 21,
                    name = A.L["Pre Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.r = r;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.g = g;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.b = b;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.a = a;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColorHex = "|c"..A:PercentToHex(r, g, b, a);
                    end,
                    get = function()
                        return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.r,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.g,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.b,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].preColor.a;
                    end
                },
                postColor =
                {
                    order = 22,
                    name = A.L["Post Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.r = r;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.g = g;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.b = b;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.a = a;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColorHex = "|c"..A:PercentToHex(r, g, b, a);
                    end,
                    get = function()
                        return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.r,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.g,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.b,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].postColor.a;
                    end
                },
                separatorColor =
                {
                    order = 23,
                    name = A.L["Separator Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.r = r;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.g = g;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.b = b;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.a = a;
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColorHex = "|c"..A:PercentToHex(r, g, b, a);
                    end,
                    get = function()
                        return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.r,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.g,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.b,
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k].separatorColor.a;
                    end
                },
                header30 =
                {
                    order = 30,
                    name = A.L["Delete"],
                    type = "header"
                },
                deleteToggle =
                {
                    order = 31,
                    name = A.L["Delete"],
                    desc = A.L["Activate the delete button."],
                    type = "toggle",
                    set = function(info, val) deleteDbObject[k] = val; end,
                    get = function(info) return deleteDbObject[k]; end
                },
                deleteExecute =
                {
                    order = 32,
                    name = A.L["Delete"],
                    desc = A.L["Delete the %s Data Broker object.\n\nThis will require reloading the user interface.\n\n|cffff3333No more warning is given beyond this point."]:format(k),
                    disabled = function() return not deleteDbObject[k]; end,
                    type = "execute",
                    func = function() 
                        A.db.global.customDisplays[k] = nil;
                        ReloadUI();
                    end
                }
            }
        };

        order = order + 1;
    end

    order = 0;
    local dbObjectsSelect = A:CreateDbObjectsSelect();
    for k,v in A:PairsByKeys(A.builtInStats) do
        options.args.builtInStats.args[v.type].args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                enable =
                {
                    order = 0,
                    name = A.L["Enable"],
                    --desc = A.L["Enable %s on the data broker display.\n\nCalculation:\n|cff4e5780%s"]:format(k, v.calc),
                    desc = function()
                        if ( v.note ) then
                            return A.L["Enable %s on the data broker display.\n\nNote: %s\n\nCalculation:\n|cff4e5780%s"]:format(k, v.note, v.calc);
                        else
                            return A.L["Enable %s on the data broker display.\n\nCalculation:\n|cff4e5780%s"]:format(k, v.calc);
                        end
                    end,
                    type = "toggle",
                    set = function(info, val)
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            A.db.profile.builtInStats[A.playerSpec][k] =
                            {
                                enabled = val
                            };
                        else
                            A.db.profile.builtInStats[A.playerSpec][k].enabled = val;
                        end
                    end,
                    get = function(info)
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.builtInStats[A.playerSpec][k].enabled;
                    end,
                },
                dataBrokerObject =
                {
                    order = 1,
                    name = A.L["Data Broker Object"],
                    desc = A.L["Choose on which Data Broker object to display %s."]:format(k),
                    type = "select",
                    values = dbObjectsSelect,
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.builtInStats[A.playerSpec][k].enabled;
                    end,
                    set = function(info, val) A.db.profile.builtInStats[A.playerSpec][k].dbObject = val; end,
                    get = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].dbObject ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].dbObject;
                        else
                            return A.L["Default"];
                        end
                    end,
                },
                configSelect =
                {
                    order = 2,
                    name = A.L["Configuration"],
                    desc = A.L["Select the configuration category."],
                    type = "select",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.builtInStats[A.playerSpec][k].enabled;
                    end,
                    values = configBuiltInSelect,
                    get = function()
                        if ( buitInConfig[k] ) then
                            return buitInConfig[k];
                        else
                            return "none";
                        end
                    end,
                    set = function(info, val) buitInConfig[k] = val; end,
                },
                displayHeader =
                {
                    order = 100,
                    name = A.L["Display"],
                    type = "header",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                enableIcon =
                {
                    order = 101,
                    name = A.L["Enable Icon"],
                    desc = A.L["Enable the icon to be displayed before the statistic"],
                    type = "toggle",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.profile.builtInStats[A.playerSpec][k].iconEnabled = val; end,
                    get = function(info)
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.builtInStats[A.playerSpec][k].iconEnabled;
                    end,
                },
                selectIcon =
                {
                    order = 102,
                    name = function() return A.L["Select Icon"]; end,
                    --desc = A.L["Select the Data Broker object main icon.\n\n|cFFC79C6ELeft-Click:|r Default icon list (Macro frame).\n|cFFC79C6EShift+Left-Click:|r Item macro list (Guild bank frame)."],
                    desc = A.L["Select the Data Broker object main icon."],
                    type = "execute",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    disabled = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.builtInStats[A.playerSpec][k].iconEnabled;
                    end,
                    image = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].icon ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].icon, 36, 36;
                        else
                            return A.defaultStatIcon, 36, 36;
                        end
                    end,
                    func = function()
                        if ( not BrokerStatisticsPopupFrame ) then A:IconsFrame(); end

                        if ( not BrokerStatisticsPopupFrame:IsVisible() ) then
                            BrokerStatisticsPopupFrame:ClearAllPoints();
                            BrokerStatisticsPopupFrame:SetPoint("CENTER", GetMouseFocus(), "CENTER", 150, 0);

                            --if ( IsShiftKeyDown() ) then BrokerStatisticsPopupFrame.iconList = 1; end
                            BrokerStatisticsPopupFrame.currentTexture = A.db.profile.builtInStats[A.playerSpec][k].icon;
                            BrokerStatisticsPopupFrame.dbEdit = A.db.profile.builtInStats[A.playerSpec][k];

                            BrokerStatisticsPopupFrame:Show();
                            A:PopupScrollUpdate();
                        end
                    end,
                },
                lineBreak10 =
                {
                    order = 110,
                    name = "",
                    type = "description",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    width = "full",
                },
                color =
                {
                    order = 111,
                    name = A.L["Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.profile.builtInStats[A.playerSpec][k].r = r;
                        A.db.profile.builtInStats[A.playerSpec][k].g = g;
                        A.db.profile.builtInStats[A.playerSpec][k].b = b;
                        A.db.profile.builtInStats[A.playerSpec][k].a = a;
                        A.db.profile.builtInStats[A.playerSpec][k].colorHex = "|c"..A:PercentToHex(r, g, b, a);
                    end,
                    get = function()
                        if ( A.db.profile.builtInStats[A.playerSpec][k] ) then
                            if ( A.db.profile.builtInStats[A.playerSpec][k].r ) then
                                return A.db.profile.builtInStats[A.playerSpec][k].r,
                                    A.db.profile.builtInStats[A.playerSpec][k].g,
                                    A.db.profile.builtInStats[A.playerSpec][k].b,
                                    A.db.profile.builtInStats[A.playerSpec][k].a;
                            else
                                A.db.profile.builtInStats[A.playerSpec][k].r = 1;
                                A.db.profile.builtInStats[A.playerSpec][k].g = 1;
                                A.db.profile.builtInStats[A.playerSpec][k].b = 1;
                                A.db.profile.builtInStats[A.playerSpec][k].a = 1;
                            end
                        end

                        return 1, 1 , 1, 1;
                    end,
                },
                lineBreak20 =
                {
                    order = 120,
                    name = "",
                    type = "description",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    width = "full",
                },
                pre =
                {
                    order = 121,
                    name = A.L["Pre"],
                    desc = A.L["The text to display before the statistic, ex: \"AP: \".\n\nLeave it blank for the default one (%s)."]:format(v.pre or A.L["None"]),
                    type = "input",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        if ( not val or val == "" ) then
                            A.db.profile.builtInStats[A.playerSpec][k].pre = nil;
                        else
                            A.db.profile.builtInStats[A.playerSpec][k].pre = val;
                        end
                    end,
                    get = function(info, val)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].pre;
                        else
                            return nil;
                        end
                    end,
                },
                post =
                {
                    order = 122,
                    name = A.L["Post"],
                    desc = A.L["The text to display after the statistic, ex: \"%%\"\nNote: this is displayed after any stat in multiple mode.\n\nLeave it blank for the default one (%s)."]:format(v.post or A.L["None"]),
                    type = "input",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "display" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        if ( not val or val == "" ) then
                            A.db.profile.builtInStats[A.playerSpec][k].post = nil;
                        else
                            A.db.profile.builtInStats[A.playerSpec][k].post = val;
                        end
                    end,
                    get = function(info, val)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].post;
                        else
                            return nil;
                        end
                    end,
                },
                alertHeader =
                {
                    order = 200,
                    name = A.L["Alert"],
                    type = "header",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                alertSelect =
                {
                    order = 201,
                    name = A.L["Alert"],
                    desc = A.L["Choose which alert to display for %s."]:format(k),
                    type = "select",
                    values = function()
                        if ( A.db.profile.builtInStats[A.playerSpec][k].alert ) then
                            return A:CreateAlertsSelect(A.db.profile.builtInStats[A.playerSpec][k].alert);
                        else
                            return A:CreateAlertsSelect();
                        end
                    end,
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        A:HideAlertFrames(A.db.profile.builtInStats[A.playerSpec][k].alert);
                        A.db.profile.builtInStats[A.playerSpec][k].alert = val;
                    end,
                    get = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].alert ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].alert;
                        else
                            return "none";
                        end
                    end,
                },
                alertModeSelect =
                {
                    order = 202,
                    name = A.L["Mode"],
                    desc = A.L["Select the alert behavior.\n\n|cFFC79c6eNormal:|r The alert will be displayed when the statistic value reach the specified value and until the statistic value drop below the specified value.\n\n|cFFC79c6eProc:|r The alert will be displayed when the statistic value reach the specified value for the specified time."],
                    type = "select",
                    values = configAlertModeSelect,
                    disabled = function()
                        if ( A.db.profile.builtInStats[A.playerSpec][k]
                        and A.db.profile.builtInStats[A.playerSpec][k].alert
                        and A.db.profile.builtInStats[A.playerSpec][k].alert ~= "none" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        if ( not A.db.profile.builtInStats[A.playerSpec][k].alertTimer ) then
                            A.db.profile.builtInStats[A.playerSpec][k].alertTimer = 3;
                        end

                        A.db.profile.builtInStats[A.playerSpec][k].alertMode = val; end,
                    get = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].alertMode ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].alertMode;
                        else
                            return "normal";
                        end
                    end,
                },
                alertValue =
                {
                    order = 203,
                    name = A.L["Value"],
                    desc = A.L["Enter the alert threshold."],
                    type = "input",
                    disabled = function()
                        if ( A.db.profile.builtInStats[A.playerSpec][k]
                        and A.db.profile.builtInStats[A.playerSpec][k].alert
                        and A.db.profile.builtInStats[A.playerSpec][k].alert ~= "none" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        local check = tonumber(val);

                        if ( check ) then
                            A.db.profile.builtInStats[A.playerSpec][k].alertValue = check;
                        else
                            A:Message(A.L["Alert value can only be a number."], 1, 1);
                        end
                    end,
                    get = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].alertValue ) then
                            return tostring(A.db.profile.builtInStats[A.playerSpec][k].alertValue);
                        else
                            return "";
                        end
                    end,
                },
                alertTimer =
                {
                    order = 204,
                    name = A.L["Timer"],
                    desc = A.L["Select the time to hide the alert."],
                    type = "range",
                    disabled = function()
                        if ( A.db.profile.builtInStats[A.playerSpec][k]
                        and A.db.profile.builtInStats[A.playerSpec][k].alert
                        and A.db.profile.builtInStats[A.playerSpec][k].alert ~= "none"
                        and A.db.profile.builtInStats[A.playerSpec][k].alertMode == "proc" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    min = 1,
                    max = 20,
                    step = 1,
                    set = function(info, val) A.db.profile.builtInStats[A.playerSpec][k].alertTimer = val; end,
                    get = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].alertTimer ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].alertTimer;
                        else
                            return 3;
                        end
                    end,
                },
                orderHeader =
                {
                    order = 300,
                    name = A.L["Order"],
                    type = "header",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "order" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                orderDescription =
                {
                    order = 301,
                    name = function()
                        local dbObject;

                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].dbObject ) then
                            dbObject = A.db.profile.builtInStats[A.playerSpec][k].dbObject;
                        else
                            dbObject = A.L["Default"];
                        end

                        return orderDescription[dbObject];
                    end,
                    type = "description",
                    width = "full",
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "order" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                orderSelect =
                {
                    order = 302,
                    name = A.L["Order"],
                    desc = A.L["Choose the order of %s."]:format(k),
                    type = "select",
                    values = orderSelect,
                    hidden = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.builtInStats[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( buitInConfig[k] == "order" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.profile.builtInStats[A.playerSpec][k].order = val; end,
                    get = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].order ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].order;
                        else
                            return nil;
                        end
                    end,
                },
            },
        };

        order = order + 1;
    end

    order = 1;
    for k,v in A:PairsByKeys(A.db.global.custom) do
        local calc = v.calc or A.L["Not Available"];

        options.args.customStats.args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                -- header1 =
                -- {
                    -- order = 0,
                    -- name = A.L["Main"],
                    -- type = "header",
                -- },
                enabled =
                {
                    order = 1,
                    name = A.L["Enable"],
                    desc = A.L["Enable %s on the data broker display.\n\nCalculation:\n|cff4e5780%s"]:format(k, calc),
                    type = "toggle",
                    set = function(info, val)
                        A.db.profile.custom[A.playerSpec][k].enabled = val;
                    end,
                    get = function(info)
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then
                            A.db.profile.custom[A.playerSpec][k] = {};

                            return nil;
                        end

                        return A.db.profile.custom[A.playerSpec][k].enabled;
                    end,
                },
                deleteToggle =
                {
                    order = 2,
                    name = A.L["Delete"],
                    desc = A.L["Activate the delete button."],
                    type = "toggle",
                    set = function(info, val) deleteCustom[k] = val; end,
                    get = function(info) return deleteCustom[k]; end
                },
                deleteExecute =
                {
                    order = 3,
                    name = A.L["Delete"],
                    desc = A.L["Delete the %s custom statistic.\n|cffff3333No more warning is given beyond this point."]:format(k),
                    hidden = function() return not deleteCustom[k]; end,
                    type = "execute",
                    func = function()
                        A.db.global.custom[k] = nil;
                        A:DelUnusedEntry();
                    end
                },
                lineBreak10 =
                {
                    order = 10,
                    name = "",
                    type = "description",
                    width = "full"
                },
                dataBrokerObject =
                {
                    order = 11,
                    name = A.L["Data Broker Object"],
                    desc = A.L["Choose on which Data Broker object to display %s."]:format(k),
                    type = "select",
                    values = dbObjectsSelect,
                    set = function(info, val) A.db.profile.custom[A.playerSpec][k].dbObject = val; end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k] and A.db.profile.custom[A.playerSpec][k].dbObject ) then
                            return A.db.profile.custom[A.playerSpec][k].dbObject;
                        else
                            return A.L["Default"];
                        end
                    end
                },
                configSelect =
                {
                    order = 12,
                    name = A.L["Configuration"],
                    desc = A.L["Select the configuration category."],
                    type = "select",
                    values = configCustomSelect,
                    get = function()
                        if ( customConfig[k] ) then
                            return customConfig[k];
                        else
                            return "none";
                        end
                    end,
                    set = function(info, val) customConfig[k] = val; end,
                },
                optionsHeader100 = -- Options
                {
                    order = 100,
                    name = A.L["Options"],
                    type = "header",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                enableIcon =
                {
                    order = 101,
                    name = A.L["Enable Icon"],
                    desc = A.L["Enable the icon to be displayed before the statistic"],
                    type = "toggle",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.profile.custom[A.playerSpec][k].iconEnabled = val; end,
                    get = function(info)
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.custom[A.playerSpec][k].iconEnabled;
                    end
                },
                selectIcon =
                {
                    order = 102,
                    name = function()
                        return A.L["Select Icon"]
                    end,
                    --desc = A.L["Select the Data Broker object main icon.\n\n|cFFC79C6ELeft-Click:|r Default icon list (Macro frame).\n|cFFC79C6EShift+Left-Click:|r Item macro list (Guild bank frame)."],
                    desc = A.L["Select the Data Broker object main icon."],
                    type = "execute",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    disabled = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.custom[A.playerSpec][k].iconEnabled;
                    end,
                    image = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k] and A.db.profile.custom[A.playerSpec][k].icon ) then
                            return A.db.profile.custom[A.playerSpec][k].icon, 36, 36;
                        else
                            return A.defaultStatIcon, 36, 36;
                        end
                    end,
                    func = function()
                        if ( not BrokerStatisticsPopupFrame ) then A:IconsFrame(); end

                        if ( not BrokerStatisticsPopupFrame:IsVisible() ) then
                            BrokerStatisticsPopupFrame:ClearAllPoints();
                            BrokerStatisticsPopupFrame:SetPoint("CENTER", GetMouseFocus(), "CENTER", 150, 0);

                            --if ( IsShiftKeyDown() ) then BrokerStatisticsPopupFrame.iconList = 1; end
                            BrokerStatisticsPopupFrame.currentTexture = A.db.profile.custom[A.playerSpec][k].icon;
                            BrokerStatisticsPopupFrame.dbEdit = A.db.profile.custom[A.playerSpec][k];

                            BrokerStatisticsPopupFrame:Show();
                            A:PopupScrollUpdate();
                        end
                    end
                },
                lineBreak110 =
                {
                    order = 110,
                    name = "",
                    type = "description",
                    width = "full",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                color =
                {
                    order = 111,
                    name = A.L["Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.profile.custom[A.playerSpec][k].r = r;
                        A.db.profile.custom[A.playerSpec][k].g = g;
                        A.db.profile.custom[A.playerSpec][k].b = b;
                        A.db.profile.custom[A.playerSpec][k].a = a;
                        A.db.profile.custom[A.playerSpec][k].colorHex = "|c"..A:PercentToHex(r, g, b, a);
                    end,
                    get = function()
                        if ( A.db.profile.custom[A.playerSpec][k] ) then
                            if ( A.db.profile.custom[A.playerSpec][k].r ) then
                                return A.db.profile.custom[A.playerSpec][k].r,
                                    A.db.profile.custom[A.playerSpec][k].g,
                                    A.db.profile.custom[A.playerSpec][k].b,
                                    A.db.profile.custom[A.playerSpec][k].a;
                            else
                                A.db.profile.custom[A.playerSpec][k].r = 1;
                                A.db.profile.custom[A.playerSpec][k].g = 1;
                                A.db.profile.custom[A.playerSpec][k].b = 1;
                                A.db.profile.custom[A.playerSpec][k].a = 1;
                            end
                        end

                        return 1, 1 , 1, 1;
                    end,
                },
                lineBreak120 =
                {
                    order = 120,
                    name = "",
                    type = "description",
                    width = "full",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                pre =
                {
                    order = 121,
                    name = A.L["Pre"],
                    desc = A.L["The text to display before the statistic, ex: \"AP: \"."],
                    type = "input",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.global.custom[k].pre = val; end,
                    get = function(info, val) return A.db.global.custom[k].pre; end,
                },
                post =
                {
                    order = 122,
                    name = A.L["Post"],
                    desc = A.L["The text to display after the statistic, ex: \"%\"\nNote: this is displayed after any stat in multiple mode."],
                    type = "input",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.global.custom[k].post = val; end,
                    get = function(info, val) return A.db.global.custom[k].post; end,
                },
                lineBreak130 =
                {
                    order = 130,
                    name = "",
                    type = "description",
                    width = "full",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                multiple =
                {
                    order = 131,
                    name = A.L["Multiple"],
                    desc = A.L["Tell the display engine there is more than one statistic returned."],
                    type = "toggle",
                    disabled = function() return not A.db.global.custom[k].separator; end,
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.global.custom[k].multiple = val; end,
                    get = function(info) return A.db.global.custom[k].multiple; end,
                },
                separator =
                {
                    order = 132,
                    name = A.L["Separator"],
                    desc = A.L["The separator used in multiple mode."],
                    type = "input",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        if ( val == "" ) then
                            val = nil;
                            A.db.global.custom[k].multiple = nil;
                        end
                        A.db.global.custom[k].separator = val; end,
                    get = function(info, val) return A.db.global.custom[k].separator; end,
                },
                lineBreak140 =
                {
                    order = 140,
                    name = "",
                    type = "description",
                    width = "full",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                round =
                {
                    order = 141,
                    name = A.L["Round"],
                    desc = A.L["Round the result to the specified decimal places."],
                    type = "select",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    values = { [1000] = A.L["Disabled"], [0] = 0, [1] = 1, [2] = 2, [3] = 3, [4] = 4, [5] = 5 },
                    set = function(info, val)
                        if ( val == 1000 ) then
                            A.db.global.custom[k].round = nil;
                        else
                            A.db.global.custom[k].round = val;
                        end
                    end,
                    get = function(info)
                        if ( A.db.global.custom[k].round ) then
                            return A.db.global.custom[k].round;
                        else
                            return 1000;
                        end
                    end,
                },
                calc =
                {
                    order = 142,
                    name = A.L["Calculation Code"],
                    desc = A.L["The code used to calculate the statistic."],
                    type = "input",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "options" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    multiline = 1,
                    width = "full",
                    set = function(info, val) A.db.global.custom[k].calc = val; end,
                    get = function(info, val) return A.db.global.custom[k].calc; end,
                },
                shareHeader200 = -- Share
                {
                    order = 200,
                    name = A.L["Share"],
                    type = "header",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "share" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                sharePlayer =
                {
                    order = 201,
                    name = A.L["Player"],
                    desc = A.L["Send %s to another player."]:format(k),
                    type = "execute",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "share" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    func = function()
                        local pop = StaticPopup_Show("BrokerStatsSendCustom");
                        if (pop) then pop.customStat = k; end
                    end,
                },
                shareGuild =
                {
                    order = 202,
                    name = A.L["Guild"],
                    desc = A.L["Send %s to your guild."]:format(k),
                    type = "execute",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "share" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    func = function() A:SendStat("GUILD", k); end,
                },
                shareRaid =
                {
                    order = 203,
                    name = A.L["Raid"],
                    desc = A.L["Send %s to your raid. It will send it to your group if you are in a party."]:format(k),
                    type = "execute",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "share" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    func = function() A:SendStat("RAID", k); end,
                },
                shareBg =
                {
                    order = 204,
                    name = A.L["Battleground"],
                    desc = A.L["Send %s to your battleground raid."]:format(k),
                    type = "execute",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "share" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    func = function() A:SendStat("BATTLEGROUND", k); end,
                },
                shareInput =
                {
                    order = 205,
                    name = A.L["String"],
                    desc = A.L["Use this to store this custom stat."],
                    type = "input",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "share" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    multiline = 1,
                    width = "full",
                    get = function() return k..A:TableTostring(v); end,
                    set = false,
                },
                alertHeader300 = -- Alert
                {
                    order = 300,
                    name = A.L["Alert"],
                    type = "header",
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                alertSelect =
                {
                    order = 301,
                    name = A.L["Alert"],
                    desc = A.L["Choose which alert to display for %s."]:format(k),
                    type = "select",
                    values = function()
                        if ( A.db.profile.custom[A.playerSpec][k].alert ) then
                            return A:CreateAlertsSelect(A.db.profile.custom[A.playerSpec][k].alert);
                        else
                            return A:CreateAlertsSelect();
                        end
                    end,
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        A:HideAlertFrames(A.db.profile.custom[A.playerSpec][k].alert);
                        A.db.profile.custom[A.playerSpec][k].alert = val;
                    end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k] and A.db.profile.custom[A.playerSpec][k].alert ) then
                            return A.db.profile.custom[A.playerSpec][k].alert;
                        else
                            return "none";
                        end
                    end,
                },
                alertModeSelect =
                {
                    order = 302,
                    name = A.L["Mode"],
                    desc = A.L["Select the alert behavior.\n\n|cFFC79c6eNormal:|r The alert will be displayed when the statistic value reach the specified value and until the statistic value drop below the specified value.\n\n|cFFC79c6eProc:|r The alert will be displayed when the statistic value reach the specified value for the specified time."],
                    type = "select",
                    values = configAlertModeSelect,
                    disabled = function()
                        if ( A.db.profile.custom[A.playerSpec][k].alert and A.db.profile.custom[A.playerSpec][k].alert ~= "none" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        if ( not A.db.profile.custom[A.playerSpec][k].alertTimer ) then
                            A.db.profile.custom[A.playerSpec][k].alertTimer = 3;
                        end

                        A.db.profile.custom[A.playerSpec][k].alertMode = val; end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k].alertMode ) then
                            return A.db.profile.custom[A.playerSpec][k].alertMode;
                        else
                            return "normal";
                        end
                    end,
                },
                alertValue =
                {
                    order = 303,
                    name = A.L["Value"],
                    desc = A.L["Enter the alert threshold."],
                    type = "input",
                    disabled = function()
                        if ( A.db.profile.custom[A.playerSpec][k].alert and A.db.profile.custom[A.playerSpec][k].alert ~= "none" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        local check = tonumber(val);

                        if ( check ) then
                            A.db.profile.custom[A.playerSpec][k].alertValue = check;
                        else
                            A:Message(A.L["Alert value can only be a number."], 1, 1);
                        end
                    end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k].alertValue ) then
                            return tostring(A.db.profile.custom[A.playerSpec][k].alertValue);
                        else
                            return "";
                        end
                    end,
                },
                alertTimer =
                {
                    order = 304,
                    name = A.L["Timer"],
                    desc = A.L["Select the time to hide the alert."],
                    type = "range",
                    disabled = function()
                        if ( A.db.profile.custom[A.playerSpec][k].alert ~= "none" and A.db.profile.custom[A.playerSpec][k].alertMode == "proc" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hidden = function()
                        if ( deleteCustom[k] ) then
                            return 1;
                        elseif ( customConfig[k] == "alert" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    min = 1,
                    max = 20,
                    step = 1,
                    set = function(info, val) A.db.profile.custom[A.playerSpec][k].alertTimer = val; end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k].alertTimer ) then
                            return A.db.profile.custom[A.playerSpec][k].alertTimer;
                        else
                            return 3;
                        end
                    end,
                },
                orderHeader =
                {
                    order = 400,
                    name = A.L["Order"],
                    type = "header",
                    hidden = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.custom[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( customConfig[k] == "order" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                orderSelect =
                {
                    order = 401,
                    name = A.L["Order"],
                    desc = A.L["Choose the order of %s."]:format(k),
                    type = "select",
                    values = orderSelect,
                    hidden = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then
                            return 1;
                        elseif ( not A.db.profile.custom[A.playerSpec][k].enabled ) then
                            return 1;
                        elseif ( customConfig[k] == "order" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.profile.custom[A.playerSpec][k].order = val; end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k] and A.db.profile.custom[A.playerSpec][k].order ) then
                            return A.db.profile.custom[A.playerSpec][k].order;
                        else
                            return nil;
                        end
                    end,
                },
            },
        };

        order = order + 1;
    end

    order = 1;
    for k,v in A:PairsByKeys(A.db.global.alerts) do
        options.args.alerts.args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                -- header1 =
                -- {
                    -- order = 0,
                    -- name = A.L["Main"],
                    -- type = "header",
                -- },
                deleteToggle =
                {
                    order = 1,
                    name = A.L["Delete"],
                    desc = A.L["Activate the delete button."],
                    type = "toggle",
                    set = function(info, val) deleteAlert[k] = val; end,
                    get = function(info) return deleteAlert[k]; end,
                },
                deleteExecute =
                {
                    order = 2,
                    name = A.L["Delete"],
                    desc = A.L["Delete the %s alert.\n|cffff3333No more warning is given beyond this point."]:format(k),
                    disabled = function() return not deleteAlert[k]; end,
                    hidden = function() return not deleteAlert[k]; end,
                    type = "execute",
                    func = function()
                        A.db.global.alerts[k] = nil;
                        A:HideAlertFrames(k);
                        A:DelUnusedEntry();
                        deleteAlert[k] = nil;
                    end,
                },
                configSelect =
                {
                    order = 3,
                    name = A.L["Configuration"],
                    desc = A.L["Select the configuration category."],
                    type = "select",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        else
                            return nil;
                        end
                    end,
                    values = configAlertSelect,
                    get = function()
                        if ( alertConfig[k] ) then
                            return alertConfig[k];
                        else
                            return "none";
                        end
                    end,
                    set = function(info, val) alertConfig[k] = val; end,
                },
                testExecute =
                {
                    order = 4,
                    name = A.L["Test"],
                    desc = A.L["Test the %s alert.\nHiding it after 5 seconds."]:format(k),
                    type = "execute",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        else
                            return nil;
                        end
                    end,
                    disabled = function() return A.db.global.alerts[k].alertConfigMode; end,
                    func = function()
                        if ( not A.alertTest ) then
                            A:SetAlert(k);
                            A:AlertTest(k);
                        end
                    end,
                },
                usedBy =
                {
                    order = 5,
                    name = function()
                        local usedBy = A:IsAlertInUse(k);

                        return A.L["Alert |cffffd100%s|r is used by statistic: |cffffd100%s|r."]:format(k, usedBy or A.L["None"]);
                    end,
                    type = "description",
                    width = "full",
                },
                header30 =
                {
                    order = 30,
                    name = A.L["Texture"],
                    type = "header",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                enableTexture =
                {
                    order = 31,
                    name = A.L["Enable Texture"],
                    desc = A.L["Enable texture options for this alert"],
                    type = "toggle",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        A.db.global.alerts[k].enableTexture = val;
                        if ( not val and A.db.global.alerts[k].alertConfigMode ) then
                            A.db.global.alerts[k].alertConfigMode = nil;
                        end
                        A:SetAlert(k);
                    end,
                    get = function(info) return A.db.global.alerts[k].enableTexture; end,
                },
                enableConfig =
                {
                    order = 32,
                    name = A.L["Enable Config"],
                    desc = A.L["Enable the options for this alert"],
                    type = "toggle",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val)
                        A.db.global.alerts[k].alertConfigMode = val;
                        A:SetAlert(k);
                    end,
                    get = function(info)
                        if ( A.db.global.alerts[k].alertConfigMode ) then
                            A:SetAlert(k);
                            return 1;
                        else
                            return nil;
                        end
                    end,
                },
                sizeX =
                {
                    order = 33,
                    name = A.L["Width"],
                    desc = A.L["Set the frame width."],
                    type = "range",
                    disabled = function() return not A.db.global.alerts[k].alertConfigMode; end,
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    min = 16,
                    max = 256,
                    step = 1,
                    width = "full",
                    set = function(info, val)
                        A.db.global.alerts[k].x = val
                        A:SetAlert(k);
                    end,
                    get = function(info) return A.db.global.alerts[k].x; end
                },
                sizeY =
                {
                    order = 34,
                    name = A.L["Height"],
                    desc = A.L["Set the frame height."],
                    type = "range",
                    disabled = function() return not A.db.global.alerts[k].alertConfigMode; end,
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    min = 16,
                    max = 256,
                    step = 1,
                    width = "full",
                    set = function(info, val)
                        A.db.global.alerts[k].y = val
                        A:SetAlert(k);
                    end,
                    get = function(info) return A.db.global.alerts[k].y; end
                },
                postColor =
                {
                    order = 40,
                    name = A.L["Texture Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    disabled = function() return not A.db.global.alerts[k].alertConfigMode; end,
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.global.alerts[k].r = r;
                        A.db.global.alerts[k].g = g;
                        A.db.global.alerts[k].b = b;
                        A.db.global.alerts[k].a = a;
                        A:SetAlert(k);
                    end,
                    get = function()
                        return A.db.global.alerts[k].r,
                            A.db.global.alerts[k].g,
                            A.db.global.alerts[k].b,
                            A.db.global.alerts[k].a;
                    end,
                },
                alphaModeSelect =
                {
                    order = 41,
                    name = A.L["Alpha mode"],
                    desc = A.L["Select the texture alpha mode."],
                    type = "select",
                    disabled = function() return not A.db.global.alerts[k].alertConfigMode; end,
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    values = alphamodeSelect,
                    get = function() return A.db.global.alerts[k].alphaMode; end,
                    set = function(info, val)
                        A.db.global.alerts[k].alphaMode = val;
                        A:SetAlert(k);
                    end,
                },
                animationSelect =
                {
                    order = 42,
                    name = A.L["Animation"],
                    desc = A.L["Select an animation."],
                    type = "select",
                    disabled = function() return not A.db.global.alerts[k].alertConfigMode; end,
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    values = animationSelect,
                    get = function() return A.db.global.alerts[k].anim; end,
                    set = function(info, val)
                        A.db.global.alerts[k].anim = val;
                        A:SetAlert(k);
                    end,
                },
                texturesRange =
                {
                    order = 43,
                    name = A.L["Texture selection"],
                    desc = A.L["Choose the texture to use with this alert."],
                    type = "range",
                    disabled = function() return not A.db.global.alerts[k].alertConfigMode; end,
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "texture" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    min = 1,
                    max = #A.alertTextures,
                    step = 1,
                    width = "full",
                    set = function(info, val)
                        A.db.global.alerts[k].texture = val
                        A:SetAlert(k);
                    end,
                    get = function(info) return A.db.global.alerts[k].texture; end,
                },
                header60 =
                {
                    order = 60,
                    name = A.L["Sound"],
                    type = "header",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "sound" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                },
                enableSound =
                {
                    order = 61,
                    name = A.L["Enable Sound"],
                    desc = A.L["Enable sound options for this alert"],
                    type = "toggle",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "sound" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    set = function(info, val) A.db.global.alerts[k].enableSound = val; end,
                    get = function(info) return A.db.global.alerts[k].enableSound; end,
                },
                soundSelect =
                {
                    order = 62,
                    name = A.L["Sound selection"],
                    desc = A.L["Select the alert sound."],
                    type = "select",
                    hidden = function()
                        if ( deleteAlert[k] ) then
                            return 1;
                        elseif ( alertConfig[k] == "sound" ) then
                            return nil;
                        else
                            return 1;
                        end
                    end,
                    dialogControl = 'LSM30_Sound',
                    values = A.LibSharedMedia:HashTable("sound"),
                    get = function() return A.db.global.alerts[k].sound; end,
                    set = function(info, val) A.db.global.alerts[k].sound = val; end,
                },
            },
        };

        order = order + 1;
    end

    -- Profiles
    options.args.profile = A.AceDBOptions:GetOptionsTable(A.db);
    options.args.profile.order = 1000;

    A:CreateStatisticsTable();

    return options;
end

LibStub("AceConfig-3.0"):RegisterOptionsTable("BrokerStatsConfig", A.AceConfig);
A.AceConfigDialog:SetDefaultSize("BrokerStatsConfig", 800, 500);
A.AceConfigDialog:AddToBlizOptions("BrokerStatsConfig", A.L["Broker Statistics"]);
